// Andreas Kirsch, 2010
#pragma once
#include "DXUTgui.h"

// only supports a vertical layout atm
class CDXUTDialogLayout
{
public:
	CDXUTDialogLayout( CDXUTDialog &dialog, int startX, int startY, int defaultWidth = 200, int defaultHeight = 24, int verticalSpace = 10, int horizontalSpace = 5)
		: dialog( dialog ), x( startX ), y( startY ), defaultWidth( defaultWidth ), defaultHeight( defaultHeight ), verticalSpace( verticalSpace ),
		horizontalSpace( horizontalSpace ), lineHeight( 0 ), keepLine( false ), startX( startX )
	{
	}

	void surpressNewLine() {
		keepLine = true;
	}

	HRESULT AddStatic( int ID, LPCWSTR strText, int width = -1, int height = -1, bool bIsDefault=false, CDXUTStatic** ppCreated=NULL ) {
		int realWidth = getWidth( width ), realHeight = getHeight( height );
		HRESULT hr = dialog.AddStatic( ID, strText, x, y, realWidth, realHeight, bIsDefault, ppCreated );

		advance( realWidth, realHeight );

		return hr;
	}

	HRESULT AddButton( int ID, LPCWSTR strText, int width = -1, int height = -1, UINT nHotkey=0, bool bIsDefault=false, CDXUTButton** ppCreated=NULL ) {
		int realWidth = getWidth( width ), realHeight = getHeight( height );
		HRESULT hr = dialog.AddButton( ID, strText, x, y, realWidth, realHeight, nHotkey, bIsDefault, ppCreated );

		advance( realWidth, realHeight );

		return hr;
	}

	HRESULT AddCheckBox( int ID, LPCWSTR strText, int width = -1, int height = -1, bool bChecked=false, UINT nHotkey=0, bool bIsDefault=false, CDXUTCheckBox** ppCreated=NULL ) {
		int realWidth = getWidth( width ), realHeight = getHeight( height );
		HRESULT hr = dialog.AddCheckBox( ID, strText, x, y, realWidth, realHeight, bChecked, nHotkey, bIsDefault, ppCreated );

		advance( realWidth, realHeight );

		return hr;
	}

	HRESULT AddRadioButton( int ID, UINT nButtonGroup, LPCWSTR strText, int width = -1, int height = -1, bool bChecked=false, UINT nHotkey=0, bool bIsDefault=false, CDXUTRadioButton** ppCreated=NULL ) {
		int realWidth = getWidth( width ), realHeight = getHeight( height );
		HRESULT hr = dialog.AddRadioButton( ID, nButtonGroup, strText, x, y, realWidth, realHeight, bChecked, nHotkey, bIsDefault, ppCreated );

		advance( realWidth, realHeight );

		return hr;
	}

	HRESULT AddComboBox( int ID, int width = -1, int height = -1, UINT nHotKey=0, bool bIsDefault=false, CDXUTComboBox** ppCreated=NULL ) {
		int realWidth = getWidth( width ), realHeight = getHeight( height );
		HRESULT hr = dialog.AddComboBox( ID, x, y, realWidth, realHeight, nHotKey, bIsDefault, ppCreated );

		advance( realWidth, realHeight );

		return hr;
	}

	HRESULT AddSlider( int ID, int width = -1, int height = -1, int min=0, int max=100, int value=50, bool bIsDefault=false, CDXUTSlider** ppCreated=NULL ) {
		int realWidth = getWidth( width ), realHeight = getHeight( height );
		HRESULT hr = dialog.AddSlider( ID, x, y, realWidth, realHeight, min, max, value, bIsDefault, ppCreated );

		advance( realWidth, realHeight );

		return hr;
	}

	//      AddIMEEditBox has been renamed into DXUTguiIME.cpp as CDXUTIMEEditBox::CreateIMEEditBox
	HRESULT AddEditBox( int ID, LPCWSTR strText, int width = -1, int height = -1, bool bIsDefault=false, CDXUTEditBox** ppCreated=NULL ) {
		int realWidth = getWidth( width ), realHeight = getHeight( height );
		HRESULT hr = dialog.AddEditBox( ID, strText, x, y, realWidth, realHeight, bIsDefault, ppCreated );

		advance( realWidth, realHeight );

		return hr;
	}

	HRESULT AddListBox( int ID, int width = -1, int height = -1, DWORD dwStyle=0, CDXUTListBox** ppCreated=NULL ) {
		int realWidth = getWidth( width ), realHeight = getHeight( height );
		HRESULT hr = dialog.AddListBox( ID, x, y, realWidth, realHeight, dwStyle, ppCreated );

		advance( realWidth, realHeight );

		return hr;
	}
	
	HRESULT AddControl( CDXUTControl* pControl, int width = -1, int height = -1 ) {
		int realWidth = getWidth( width ), realHeight = getHeight( height );
		HRESULT hr = dialog.AddControl( pControl );

		advance( realWidth, realHeight );

		return hr;
	}

	void setXY( int x, int y ) {
		this->x = x;
		this->y = y;
	}

	void advanceYX( int dy, int dx = 0 ) {
		x += dx;
		y += dy;
	}

	CDXUTDialog & getDialog() {
		return dialog;
	}

protected:
	int getWidth( int width ) {
		if( width == -1 ) {
			return defaultWidth;
		}
		return width;
	}

	int getHeight( int height ) {
		if( height == -1 ) {
			return defaultHeight;
		}
		return height;
	}

	void advance( int width, int height ) {
		// update lineHeight
		lineHeight = (height > lineHeight) ? height : lineHeight;

		if( !keepLine ) {
			x = startX;
			y += lineHeight + verticalSpace;
			lineHeight = 0;
		}
		else {
			x += width + horizontalSpace;
		}

		keepLine = false;
	}

protected:
	CDXUTDialog &dialog;

	const int defaultWidth, defaultHeight;
	const int verticalSpace, horizontalSpace;
	const int startX;

	bool keepLine;
	int lineHeight;

	int x, y;

public:
	static const int DEFAULT = -1;
};