//--------------------------------------------------------------------------------------
// File: fxhelper.h
//
// (FreeBSD License)
// 
// Copyright 2010 Andreas Kirsch. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice, this list of
// conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice, this list
// of conditions and the following disclaimer in the documentation and/or other materials
// provided with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY ANDREAS KIRSCH ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ANDREAS KIRSCH OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Andreas Kirsch.
//--------------------------------------------------------------------------------------

// use #define EFFECT_NAME for the name of the effect
// use #define EFFECT_FILE for the effect file name 
// use #define EFFECT_DEF_FILE for the effect file name (otherwise EFFECT_FILE is used)

// EFFECT_DEFINITION is defined if fxhelper is running on a file!

// #define EFFECT( name )

// #define TECHNIQUE( name )
// #define TECHNIQUE_( name, fxName )

// type is Matrix, Scalar, Vector, ShaderResource, etc and D3DXVECTOR3, D3DXVECTOR4, etc and float, bool, int
// #define VARIABLE( type, name )
// #define VARIABLE_( type, name, fxName )

#include <boost/preprocessor.hpp>

#ifndef EFFECT_NAME
#error EFFECT_NAME not defined!
#endif

#ifndef EFFECT_FILE
#error EFFECT_FILE not defined!
#endif

#ifndef EFFECT_DEF_FILE
#define EFFECT_DEF_FILE EFFECT_FILE
#endif

// processing active
#define EFFECT_DEFINITION

// shorthands
#define TECHNIQUE( name ) TECHNIQUE_( name, fxName )
#define VARIABLE( type, name )	VARIABLE_( type, name, name )

#ifndef EFFECT_PREAMBLE
#include "DXUT.h"
#include "SDKmisc.h"
#else
EFFECT_PREAMBLE
#endif

#define EFFECT_CLASSNAME BOOST_PP_CAT( Effect, EFFECT_NAME )

	struct EFFECT_CLASSNAME {
		ID3D10Effect *effect;
		ID3D10InputLayout *inputLayout;
		EFFECT_CLASSNAME() : effect( NULL ), inputLayout( NULL ) {}
		~EFFECT_CLASSNAME() { destroy(); }

#define TECHNIQUE_( name, fxName ) \
		ID3D10EffectTechnique *name;

#define VARIABLE_( type, name, fxName ) \
	_VAR_TYPE_CPP( type ) *name;

#define _VAR_TYPE_CPP_MAP( type ) _VAR_TYPE_CPP_MAP_##type
#define _VAR_TYPE_CPP_MAP_ShaderResource	ShaderResource
#define _VAR_TYPE_CPP_MAP_Matrix			Matrix
#define _VAR_TYPE_CPP_MAP_Vector			Vector
#define _VAR_TYPE_CPP_MAP_D3DXVECTOR3		Vector
#define _VAR_TYPE_CPP_MAP_D3DXVECTOR4		Vector
#define _VAR_TYPE_CPP_MAP_Scalar			Scalar
#define _VAR_TYPE_CPP_MAP_float				Scalar
#define _VAR_TYPE_CPP_MAP_int				Scalar
#define _VAR_TYPE_CPP_MAP_bool				Scalar

#define _VAR_TYPE_CPP(type) BOOST_PP_CAT( BOOST_PP_CAT( ID3D10Effect, _VAR_TYPE_CPP_MAP( type ) ), Variable )

#include EFFECT_DEF_FILE

#undef TECHNIQUE_
#undef VARIABLE_

	HRESULT init( ID3D10Device *device, const D3D10_INPUT_ELEMENT_DESC *inputLayoutDescs, int numInputLayoutDescs ) {
		HRESULT hr;

		// Read the D3DX effect file
		WCHAR path[MAX_PATH];
		V_RETURN( DXUTFindDXSDKMediaFileCch( path, MAX_PATH, BOOST_PP_CAT( L, EFFECT_FILE ) ) );
		DWORD dwShaderFlags = D3D10_SHADER_ENABLE_STRICTNESS;
#if defined( DEBUG ) || defined( _DEBUG )
		// Set the D3D10_SHADER_DEBUG flag to embed debug information in the shaders.
		// Setting this flag improves the shader debugging experience, but still allows
		// the shaders to be optimized and to run exactly the way they will run in
		// the release configuration of this program.
		dwShaderFlags |= D3D10_SHADER_DEBUG;
#endif
		V_RETURN( D3DX10CreateEffectFromFile( path, NULL, NULL, "fx_4_0", dwShaderFlags, 0, device, NULL,
			NULL, &effect, NULL, NULL ) );

#define TECHNIQUE_( name, fxName ) \
		name = effect->GetTechniqueByName( #fxName );

#define VARIABLE_( type, name, fxName ) \
		name = effect->GetVariableByName( #fxName )->BOOST_PP_CAT( As, _VAR_TYPE_CPP_MAP( type ) )();

#include EFFECT_DEF_FILE

#undef TECHNIQUE_
#undef VARIABLE_

		if( inputLayoutDescs && numInputLayoutDescs ) {
			// Create the input layout
			D3D10_PASS_DESC PassDesc;
			effect->GetTechniqueByIndex(0)->GetPassByIndex( 0 )->GetDesc( &PassDesc );
			V_RETURN( device->CreateInputLayout( inputLayoutDescs, numInputLayoutDescs, PassDesc.pIAInputSignature,
				PassDesc.IAInputSignatureSize, &inputLayout ) );
		}
		else {
			inputLayout = NULL;
		}

		return S_OK;
	}

	void destroy() {
		SAFE_RELEASE( effect );
		SAFE_RELEASE( inputLayout );
	}

// setter methods
#define TECHNIQUE_( name, fxName )

#define VARIABLE_( type, name, fxName ) _VAR_SET_CPP_##type( name )

#define _VAR_SET_CPP_ARRAY( type, name, accessExpression, accessType ) \
	void setArray_##name##( accessType* *array, int count, int offset = 0) { \
		name->Set##type##Array( accessExpression, offset, count ); \
	}


#define _VAR_SET_CPP_ShaderResource( name )	\
	void set_##name( ID3D10ShaderResourceView *resource ) { \
		name->SetResource( resource ); \
	}\
	\
	void setArray_##name( ID3D10ShaderResourceView* *resources, int count, int offset = 0) { \
		name->SetResourceArray( resources, offset, count ); \
	}	

#define _VAR_SET_CPP_Matrix( name )	\
	void set_##name( const D3DXMATRIX &matrix ) { \
		name->SetMatrix( (float*) (const float*) matrix  ); \
	}

#define _VAR_SET_CPP_float( name )	\
	void set_##name( float value ) { \
		name->SetFloat( value ); \
	}

#define _VAR_SET_CPP_int( name )	\
	void set_##name( int value ) { \
		name->SetInt( value ); \
	}

#define _VAR_SET_CPP_bool( name )	\
	void set_##name( bool value ) { \
		name->SetBool( value ); \
	}

#define _VAR_SET_CPP_Scalar( name )	\
	_VAR_SET_CPP_float( name )\
	_VAR_SET_CPP_int( name )\
	_VAR_SET_CPP_bool( name )

#define _VAR_SET_CPP_D3DXVECTOR3( name ) \
	void set_##name( const D3DXVECTOR3 &vector ) { \
		name->SetFloatVector( (float*) (const float*) vector ); \
	}
	
#define _VAR_SET_CPP_D3DXVECTOR4( name ) \
	void set_##name( const D3DXVECTOR4 &vector ) { \
		name->SetFloatVector( (float*) (const float*) vector ); \
	}

#define _VAR_SET_CPP_Vector( name ) \
	_VAR_SET_CPP_D3DXVECTOR3( name ) \
	_VAR_SET_CPP_D3DXVECTOR4( name )

#include EFFECT_DEF_FILE

#undef TECHNIQUE_
#undef VARIABLE_
};

#undef EFFECT_DEFINITION
