//--------------------------------------------------------------------------------------
// File: sphericalharmonics.h
//
// (FreeBSD License)
// 
// Copyright 2010 Andreas Kirsch. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice, this list of
// conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice, this list
// of conditions and the following disclaimer in the documentation and/or other materials
// provided with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY ANDREAS KIRSCH ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ANDREAS KIRSCH OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Andreas Kirsch.
//--------------------------------------------------------------------------------------
#pragma once

#include <assert.h>
#define _USE_MATH_DEFINES
#include <math.h>

#include "DXUT.h"

namespace SH {
	const float band0Factor = 0.282094792f;
	const float band1Factor = 0.488602512f;

	D3DXVECTOR4 evalDirection( const D3DXVECTOR3 &direction ) {
		assert( fabs( D3DXVec3LengthSq( &direction ) - 1.0f ) < 0.1f );

		D3DXVECTOR4 coeff;
#if 0
		// l = 0, m = 0
		coeff.x = 0.5f / sqrt( M_PI );
		
		// l = 1
		double scaleFactor = 0.5f * sqrt( 3 / M_PI );
		// l = 1, m = -1
		coeff.y = -scaleFactor * direction.y;
		// l = 1, m = 0
		coeff.z = scaleFactor * direction.z;
		// l = 1, m = +1
		coeff.w = -scaleFactor * direction.x;
#else
		// l = 0, m = 0
		coeff.x = band0Factor;

		// l = 1

		// l = 1, m = -1
		coeff.y = -band1Factor * direction.y;
		// l = 1, m = 0
		coeff.z = band1Factor * direction.z;
		// l = 1, m = +1
		coeff.w = -band1Factor * direction.x;
#endif

		return coeff;
	}

	// from Peter Sloan's Stupid SH Tricks
	D3DXVECTOR4 CosMult( const D3DXVECTOR4 &in )
	{
		D3DXVECTOR4 out;

		// constants to make code fit
		const float T4 = 0.2886751347f;

		out[0] = 0.25f * in[0] + T4 * in[2];
		out[1] = 0.1875f * in[1];
		out[2] = 0.375f * in[2] + T4 * in[0];
		out[3] = 0.1875f * in[3];
		return out;
	}
}

